@@ -1,9 +1,9 @@
+import ast_comments as ast
 import asyncio
 import openai
 import click
 import glob
 import os
-import ast
 import tqdm
 
 # import commentator
@@ -11,29 +11,34 @@ from . import commentator
 
 async def do_one_file(index, file, language):
     code = file.read()
+    try:
+        ast.parse(code)
+    except SyntaxError:
+        # Failed to parse.
+        return
+        
     function_count = 0
     for func_name in commentator.enumerate_functions(code):
         the_code = commentator.extract_function_source(code, func_name)
         if not (commentator.has_docstring(the_code) and commentator.has_types(the_code)):
             function_count += 1
             
-    from tqdm import tqdm
-    pbar = tqdm(total=1, dynamic_ncols=True, desc=file.name, leave=False, unit='function') # position=index, 
-    
     if function_count == 0:
-        pbar.total = 1
-        pbar.update(1)
-        pbar.set_description(f'{file.name}: no functions need commentating.')
-        # pbar.update(1)
         return
 
-    pbar.total=function_count
+    from tqdm import tqdm
+    pbar = tqdm(total=function_count, desc=file.name, leave=False, unit='function') # position=index,
+    
     (result, successes) = await commentator.commentate(file.name, code, pbar, language)
     if result:
         save_path = "backup"
         if not os.path.exists(save_path):
             os.makedirs(save_path)
         with open(os.path.join(save_path, file.name), 'w') as f:
+            # Generate import statement first.
+            import_stmt = commentator.generate_import(ast.parse(code))
+            if import_stmt:
+                code = import_stmt + "\n" + code
             f.write(code)
         with open(file.name, 'w') as f:
             f.write(result)
@@ -52,17 +57,25 @@ async def do_it(api_key, language, *files):
     file_list = list(*files)
     tasks = [do_one_file(index, file, language) for (index, file) in enumerate(file_list)]
     await asyncio.gather(*tasks)
-    
+
+def print_version(ctx, param, value):
+    import importlib.metadata
+    if not value or ctx.resilient_parsing:
+        return
+    click.echo(f"commentator version {importlib.metadata.metadata('python-commentator')['Version']}")
+    ctx.exit(0)
+
 @click.command()
 @click.argument('file', nargs=-1, type=click.File('r'))
 @click.option('--api-key', help="OpenAI key.", default=commentator.api_key(), required=False)
 @click.option('--language', help="Optionally adds translations in the (human) language of your choice.", required=False, default=None)
-def main(file, api_key, language):
+@click.option('--version', is_flag=True, callback=print_version,
+              expose_value=False, is_eager=True)
+def main(file, api_key, language, version):
     """Automatically adds comments to your code.
 
     See https://github.com/emeryberger/commentator for more information.
     """
     asyncio.run(do_it(api_key, language, file))
-    print("Commentator complete.")
 
 main()
