@@ -1,4 +1,4 @@
-import ast
+import ast_comments as ast
 import openai_async
 import asyncio
 import logging
@@ -6,37 +6,80 @@ import openai
 import os
 import sys
 import tqdm
-from typing import cast, Optional, List, Set, Tuple, Union
+
+from typing import Any, cast, Deque, DefaultDict, Dict, FrozenSet, List, Optional, Set, Tuple, Union
+import typing
+
+from . import collect_types
+from . import strip_comments
+from . import strip_types
+
 logname = 'commentator.log'
 logging.basicConfig(filename=logname, filemode='a', format='%(asctime)s,%(msecs)d %(name)s %(levelname)s %(message)s', datefmt='%H:%M:%S', level=logging.INFO)
 logging.info('Running commentator.')
 
 successful_comments = 0
 
+
+def generate_import(node):
+    """
+    Generate an import statement that imports from typing any types from
+    typing that were declared as annotations in the given AST node.
+    """
+    # Find all type annotations in the node
+    typing_classes = ["List", "Tuple", "Set", "FrozenSet", "Dict", "DefaultDict", "Deque", "Any", "Union", "Optional", "cast"]
+    types_used = collect_types.collect_types(node)
+    typing_imports = set()
+    for t in types_used:
+        for c in typing_classes:
+            if t.startswith(c):
+                typing_imports.add(c)
+                break
+    
+    # Generate the import statement
+    if typing_imports:
+        return 'from typing import ' + ', '.join(sorted(list(typing_imports)))
+    else:
+        return ''
+
 async def get_comments(programming_language: str, func_name: str, translate_text: str, the_code: str, pbar) -> Optional[openai.api_resources.Completion]:
     import httpx
-    content = f'Rewrite the following {programming_language}code by adding high-level explanatory comments as PEP 257 docstrings, and PEP 484 style type annotations. Infer what each function does, using the names, comments, and computations as hints. If there are existing comments or types, augment them rather than replacing them. If the existing comments are inconsistent with the code, correct them. Every function argument and return value should be typed if possible. Do not change any other code. {translate_text} {the_code}'
+    content = f'Add comments to the following {programming_language}code. Add both low-level and high-level explanatory comments as per-line comments starting with #, PEP 257 docstrings, and PEP 484 style type annotations. Make sure to add comments before all loops, branches, and complicated lines of code. Infer what each function does, using the names, comments, and computations as hints. If there are existing comments or types, augment them rather than replacing them. If existing comments are inconsistent with the code, correct them. Every function argument and return value should be typed. {translate_text}ONLY RETURN THE UPDATED FUNCTION: {the_code}'
     try:
         max_trials = 3
         for trial in range(max_trials):
-            completion = await openai_async.chat_complete(openai.api_key, timeout=30, payload={'model': 'gpt-3.5-turbo', 'messages': [{'role': 'system', 'content': 'You are a {programming_language}programming assistant who ONLY responds with blocks of code. You never respond with text. Just code, starting with ``` and ending with ```.', 'role': 'user', 'content': content}]})
+            completion = await openai_async.chat_complete(openai.api_key, timeout=30, payload={'model': 'gpt-3.5-turbo', 'messages': [{'role': 'system', 'content': 'You are a {programming_language}programming assistant who ONLY responds with blocks of commented and typed code. You never respond with text. Just code, starting with ``` and ending with ```.', 'role': 'user', 'content': content}]})
             code_block = extract_code_block(completion.json())
+            logging.info(f'PROCESSING {code_block}')
             if validated(the_code, code_block):
+                logging.info(f'Validated code block:\n-----\n{code_block}\n-----')
                 global successful_comments
                 successful_comments += 1
-                # Splice in the types and the docstring from the generated function into the original function.
+                # If the commented version is equivalent to the uncommented version, use it.
                 the_code_ast = ast.parse(the_code)
                 code_block_ast = ast.parse(code_block)
-                for node in ast.walk(the_code_ast):
-                    if isinstance(node, (ast.FunctionDef, ast.AsyncFunctionDef)):
-                        tca = node
-                        break
-                for node in ast.walk(code_block_ast):
-                    if isinstance(node, (ast.FunctionDef, ast.AsyncFunctionDef)):
-                        cba = node
-                        break
-                replace_function_annotations(tca, cba)
-                code_block = ast.unparse(tca)
+                stripped_the_code = strip_comments.strip_comments(the_code_ast)
+                stripped_the_code = strip_types.strip_types(ast.parse(stripped_the_code))
+                stripped_code_block = strip_comments.strip_comments(code_block_ast)
+                stripped_code_block = strip_types.strip_types(ast.parse(stripped_code_block))
+                if stripped_the_code == stripped_code_block:
+                    logging.info(f"COMMENTS EQUAL WUT\n=====\n{stripped_the_code}\n=====\n{stripped_code_block}")
+                    pass
+                else:
+                    logging.info(f"COMMENTS NOT EQUAL WUT\n=====\n{stripped_the_code}\n=====\n{stripped_code_block}")
+                    # Otherwise, just splice in the types and the
+                    # docstring from the generated function into the
+                    # original function.
+                    for node in ast.walk(the_code_ast):
+                        if isinstance(node, (ast.FunctionDef, ast.AsyncFunctionDef)):
+                            tca = node
+                            break
+                    for node in ast.walk(code_block_ast):
+                        if isinstance(node, (ast.FunctionDef, ast.AsyncFunctionDef)):
+                            cba = node
+                            break
+                    replace_function_annotations(tca, cba)
+                    code_block = ast.unparse(tca)
                 break
             else:
                 logging.info(f'Failed to validate:\n-----\n{code_block}\n-----')
@@ -48,6 +91,9 @@ async def get_comments(programming_language: str, func_name: str, translate_text
         import sys
         sys.exit(1)
     except Exception as e:
+        #print("OH SNAP", e)
+        #import traceback
+        #print(traceback.format_exc())
         pbar.update(1)
         return ''
     pbar.update(1)
@@ -109,7 +155,6 @@ def update_args(old_function_ast: Union[ast.FunctionDef, ast.AsyncFunctionDef],
     return old_function_ast
 test = '\ndef abs(n):\n    """ WUT """\n    # Check if integer is negative\n    if n < 0:\n        # Return the opposite sign of n (i.e., multiply n by -1)\n        return -n\n    else:\n        # Return n (which is already a positive integer or zero)\n        return n\n'
 test2 = '\ndef abs(n):\n    if n < 0:\n        return -n\n    else:\n        return n\n'
-import ast
 
 def remove_code_before_function(code: str) -> str:
     """
@@ -250,11 +295,6 @@ def extract_function_ast(program_str: str, function_name: str) -> Union[ast.Func
 
 def extract_function_source(program_str, function_name):
     return ast.unparse(extract_function_ast(program_str, function_name))
-    program_ast = ast.parse(program_str)
-    function_node = next((n for n in program_ast.body if isinstance(n, ast.FunctionDef) and n.name == function_name), None)
-    if function_node is None:
-        raise ValueError(f"No function named '{function_name}' was found")
-    return ast.unparse(function_node)
 
 def enumerate_functions(program_str: str) -> List[str]:
     """
@@ -266,7 +306,11 @@ def enumerate_functions(program_str: str) -> List[str]:
     Returns:
         A list of names of functions and async functions defined in the program.
     """
-    program_ast = ast.parse(program_str)
+    try:
+        program_ast = ast.parse(program_str)
+    except SyntaxError:
+        # Failed to parse.
+        return []
     names = [n.name for n in program_ast.body if isinstance(n, (ast.FunctionDef, ast.AsyncFunctionDef))]
     return names
 
@@ -427,7 +471,7 @@ async def commentate(filename: str, code: str, pbar, language: Optional[str]=Non
     else:
         from tqdm import tqdm
         num_items = len(the_funcs)
-        pbar.total=num_items
+        pbar.total = num_items
         # pbar = tqdm(total=num_items, desc=)
         tasks = [get_comments(programming_language, f, translate_text, extract_function_source(code, f), pbar) for f in the_funcs]
         results = await asyncio.gather(*tasks)
@@ -436,9 +480,14 @@ async def commentate(filename: str, code: str, pbar, language: Optional[str]=Non
             if not code_block:
                 continue
             code = replace_function(code, func_name, code_block)
+    import_stmt = generate_import(ast.parse(code))
+    if import_stmt:
+        code = import_stmt + '\n' + code
     global successful_comments
     return (code, successful_comments)
 
+# print(generate_import(ast.parse('x = 12\n\ndef whatever(n: float) -> Dict[str]:\n    """Creates a dictionary with a pre-defined key-value pair where key is \'X\' and value is 12.\nIf the input argument n is equal to 0.1234, then a new key-value pair is added to the dictionary \nwith key \'COOL\' and value 1.\n\n:param n: A float input value.\n:return: A dictionary containing key-value pairs."""\n    d = {\'X\': 12}\n    if n == 0.1234:\n        d[\'COOL\'] = 1\n    return d\n\ndef absolutely(n: int) -> Union[int, bool]:\n    """Return the absolute value of the input integer.\n\nArgs:\n    n (int): The input integer.\n\nReturns:\n    int: The absolute value of the input integer."""\n    if n < 0:\n        return -n\n    else:\n        return n\nprint(\'WOOT\')\n')))
+
 def api_key() -> str:
     """
     Get the API key from the environment variable 'OPENAI_API_KEY'.
